@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2012 Bastien Nocera
+   Copyright 2012 Bastien Nocera
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -62,6 +62,35 @@ struct _GeocodeLocationClass {
         GObjectClass parent_class;
 };
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GeocodeLocation, g_object_unref)
+
+/**
+ * GeocodeLocationURIScheme:
+ * @GEOCODE_LOCATION_URI_SCHEME_GEO: The 'geo' URI scheme, RFC 5870
+ *
+ * The URI scheme for this location.
+ */
+typedef enum {
+	GEOCODE_LOCATION_URI_SCHEME_GEO = 0
+} GeocodeLocationURIScheme;
+
+/**
+ * GeocodeLocationCRS:
+ * @GEOCODE_LOCATION_CRS_WGS84: CRS is World Geodetic System, standard for Earth.
+ *
+ * Coordinate Reference System Identification for a location.
+ */
+typedef enum {
+	GEOCODE_LOCATION_CRS_WGS84 = 0
+} GeocodeLocationCRS;
+
+/**
+ * GEOCODE_LOCATION_ALTITUDE_UNKNOWN:
+ *
+ * Constant representing unknown altitude.
+ */
+#define GEOCODE_LOCATION_ALTITUDE_UNKNOWN -G_MAXDOUBLE
+
 /**
  * GEOCODE_LOCATION_ACCURACY_UNKNOWN:
  *
@@ -106,15 +135,25 @@ struct _GeocodeLocationClass {
 
 #define GEOCODE_TYPE_LOCATION (geocode_location_get_type ())
 
-GeocodeLocation *geocode_location_new (gdouble latitude,
-                                       gdouble longitude,
-                                       gdouble accuracy);
+GeocodeLocation *geocode_location_new                  (gdouble latitude,
+                                                        gdouble longitude,
+                                                        gdouble accuracy);
 
 GeocodeLocation *geocode_location_new_with_description (gdouble     latitude,
                                                         gdouble     longitude,
                                                         gdouble     accuracy,
                                                         const char *description);
 
+gboolean geocode_location_equal                        (GeocodeLocation *a,
+                                                        GeocodeLocation *b);
+
+gboolean geocode_location_set_from_uri                 (GeocodeLocation *loc,
+                                                        const char      *uri,
+                                                        GError         **error);
+
+char * geocode_location_to_uri                         (GeocodeLocation *loc,
+                                                        GeocodeLocationURIScheme scheme);
+
 double geocode_location_get_distance_from              (GeocodeLocation *loca,
                                                         GeocodeLocation *locb);
 
@@ -125,6 +164,8 @@ const char *geocode_location_get_description           (GeocodeLocation *loc);
 
 gdouble geocode_location_get_latitude                  (GeocodeLocation *loc);
 gdouble geocode_location_get_longitude                 (GeocodeLocation *loc);
+gdouble geocode_location_get_altitude                  (GeocodeLocation *loc);
+GeocodeLocationCRS  geocode_location_get_crs           (GeocodeLocation *loc);
 gdouble geocode_location_get_accuracy                  (GeocodeLocation *loc);
 guint64 geocode_location_get_timestamp                 (GeocodeLocation *loc);
 
