@@ -410,10 +410,14 @@ test_pi (void)
 {
 	g_autoptr (GHashTable) params = NULL;
 	GeocodeForward *object;
+	g_autofree char *old_locale = NULL;
 	GError *error = NULL;
 	GList *res;
 	GeocodePlace *place;
 
+	old_locale = g_strdup (setlocale(LC_ALL, NULL));
+	g_assert_nonnull (setlocale (LC_ALL, "en_GB.UTF-8"));
+
 	/* The query parameters the mock server expects to receive. */
 	params = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
 	add_attr_string (params, "q", "Jack Cole Building, St Andrews");
@@ -439,6 +443,8 @@ test_pi (void)
 	g_assert_cmpstr (geocode_place_get_street_address (place), ==, "North Haugh π");
 	g_object_unref (place);
 	g_list_free (res);
+
+	setlocale (LC_ALL, old_locale);
 }
 
 static void
@@ -497,7 +503,7 @@ test_search (void)
 	got_texas = FALSE;
 
 	old_locale = g_strdup (setlocale(LC_MESSAGES, NULL));
-	setlocale (LC_MESSAGES, "en_GB.UTF-8");
+	g_assert_nonnull (setlocale (LC_MESSAGES, "en_GB.UTF-8"));
 
 	/* The query parameters the mock server expects to receive. */
 	params = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
@@ -692,7 +698,7 @@ test_locale_format (void)
 	old_locale = g_strdup (setlocale(LC_ADDRESS, NULL));
 
 	/* Set to a locale that has number after street */
-	setlocale (LC_ADDRESS, "sv_SE.utf8");
+	g_assert_nonnull (setlocale (LC_ADDRESS, "sv_SE.utf8"));
 
 	/* The query parameters the mock server expects to receive. */
 	params = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
@@ -743,7 +749,7 @@ test_locale_name (void)
 	add_attr_string (params, "bounded", "0");
 
 	/* Check Moscow's name in Czech */
-	setlocale (LC_MESSAGES, "cs_CZ.UTF-8");
+	g_assert_nonnull (setlocale (LC_MESSAGES, "cs_CZ.UTF-8"));
 
 	object = create_forward_for_string ("moscow", params,
 	                                    "locale_name.json");
